<?php
   /*
   Plugin Name: Gigatools events archiver
   Plugin URI: 
   Description: pulls gigatools events into the database, in order to archive
   Version: 1.0
   Author: 
   Author URI: 
   License: 
   */

if (!class_exists("eventsPull")) {
class eventsPull {
	var $url = 'http://gigs.gigatools.com/';
	var $op = array( 
			'gtc_name' => '',
			'gtc_type' => 'user',
			'gtc_interval' => 'daily',
			'page_id' => 0
	 	);
	var $recurrence = array('twicedaily', 'daily', 'hourly', 'never');
	var $type = array('user', 'group');
	var $eventsShortcode = 'gtc_upcomingEvents';
	var $yearArchiveShortcode = 'gtc_yearArchive';
	var $gtc_errors;
		
	function eventsPull() {
		if($options = get_option('gtc_options')){
			$this->op = $options;
		}
		if($this->op['gtc_interval']!='never') {
			add_action('gigatools_pull_events', array($this, 'doThisPeriodical'));
		}else{
			$this->unshedule_events_pull();
		}
		add_action('admin_menu', array($this, 'admin_menu'));
		add_shortcode($this->eventsShortcode, array($this, 'upcomingEventsList'));
		add_shortcode($this->yearArchiveShortcode, array($this, 'yearArchive'));
		add_action('widgets_init', array($this, 'register_gtc_widget'));
		if(!is_admin()){
			$stylePath = get_template_directory().'/gtc-templates/gtc-gigatools.css';
			if(file_exists($stylePath)){
				$styleUri = get_template_directory_uri().'/gtc-templates/gtc-gigatools.css';
			}else{
				$styleUri = plugins_url('gtc-templates/gtc-gigatools.css', __FILE__);
			}
			wp_enqueue_style('gtc-style', $styleUri);
		}
	}

	function installEventsPull(){
		$this->initTable();
		update_option('gtc_options', $this->op);
		$this->shedule_events_pull($this->op['gtc_interval']);
	}
	 
	function uninstallEventsPull(){
	 	$this->unshedule_events_pull();
	 	delete_option('gtc_options');
	}
	function register_gtc_widget(){
		require_once(GTC_PLUGIN_PATH.'/lib/widget.php');
		register_widget( 'gtc_widget' );
	}
	protected function shedule_events_pull($interval='daily'){

		if($interval!='never')
			wp_schedule_event(time(), $interval, 'gigatools_pull_events');
	}

	protected function unshedule_events_pull(){
		$timestamp = wp_next_scheduled( 'gigatools_pull_events');
		wp_unschedule_event($timestamp, 'gigatools_pull_events');
	}
	
	function getGigaUrl(){
		return $this->url.$this->op['gtc_type'].'/'.$this->op['gtc_name'].'.json';
	}

	function admin_menu(){
		//add_options_page('Gigatools settings', 'Gigatools settings', 'manage_options', 'gtc-settings', array($this, 'printAdminPage'));
		add_menu_page('Gigatools archiver settings', 'Gigatools', 'manage_options', 'gtc-settings'/*, array($this, 'printAdminPage')*/);
		add_submenu_page('gtc-settings', 'Gigatools Options', 'Gigatools Settings', 'manage_options', 'gtc-settings', array($this, 'printAdminPage'));
		add_submenu_page('gtc-settings', 'List gigatools', 'List Upcoming Events', 'manage_options', 'gtc-list-events', array($this, 'gigatoolsUpcomingPage'));
		add_submenu_page('gtc-settings', 'List gigatools', 'List Archive Events', 'manage_options', 'gtc-archive-events', array($this, 'gigatoolsArchivePage'));
		add_submenu_page('gtc-settings', 'New Event', 'Add Archive Event', 'manage_options', 'gtc-event', array($this, 'gigatoolsSingleEvent'));
		add_submenu_page('gtc-settings', 'Import Archive', 'Import Events', 'manage_options', 'gtc-import', array($this, 'gtcImportEvents'));
	}

	function printAdminPage(){
		if(isset($_POST['gtc-update'])){
			$this->update_options();
		}
	 ?>
		<div class=wrap>
			
		<h2>Edit Gigatools events archiver settings</h2>
		
		<?php
		if(empty($this->op['gtc_name']) || empty($this->op['page_id'])){
			echo $this->getMessages('no-config', 'error'); 
		} ?>
		<h3></h3>
		
		<form method="post" action=''>
			
			<?php wp_nonce_field('gtc-nonce'); ?>
			<select name="gtc_type">
				<?php foreach ($this->type as $v) {
					$selected = '';
					if($v==$this->op['gtc_type'])
						$selected = ' selected="selected"'; ?>
					<option <?php echo $selected; ?>><?php echo $v; ?></option>
				<?php } ?>
			</select>
			<label for="gtc_type">Gigatools user or group.</label><br />
			<br />
			<input type="text" value="<?php echo $this->op['gtc_name']; ?>" name="gtc_name" />
			<label for="gtc_name">* Gigatools username or groupname</label><br />
			<br />
			<select name="gtc_interval">
				<?php foreach ($this->recurrence as $v) {
					$selected = '';
					if($v==$this->op['gtc_interval'])
						$selected = ' selected="selected"'; ?>
					<option <?php echo $selected; ?>><?php echo $v; ?></option>
				<?php } ?>
			</select>
			<label for="gtc_interval">Pull events from Gigatools (daily, hourly, ...) and overwrite allready pulled events. Choose "never" to disable any updates</label><br />
			<br />
			<?php echo wp_dropdown_pages(array( 'selected'=>$this->op['page_id'], 'echo' => 0,  'show_option_none'=>' ', 'show_option_none_value'=>0)); ?>
			<label><sup>*</sup> Gigatools events and archive page.</label><br />
			<br />
			<input type="submit" value="<?php _e('Save'); ?>" name="gtc-save" id="" class="button-secondary action" /><br />
			<input type="hidden" value="1" name="gtc-update" />
			<br />
			<sup>*</sup> : Required fields.<br />
			Note*: If you would like to customize the look of your events, copy \gtc-templates\ from the plugin directory, <br />
			and paste into your active theme directory. <br />
			<br />
			<?php if($this->op['gtc_name']){ ?>
				<input type="submit" value="<?php _e('Pull events'); ?>" name="gtc-rebuild" id="" class="button-secondary action" /><br />
				<label>Rebuild events manually.</label><br />
			<?php } ?>
		</form>
		</div>
	<?php
	}

	function update_options(){
		if(check_admin_referer('gtc-nonce')){
			$changes = false;
			$changeCron = false;
			$changePage = false;
			if(isset($_POST['gtc-rebuild'])) {
				$this->doThisPeriodical();
			}
			if(isset($_POST['gtc_name']) && $this->op['gtc_name'] != $_POST['gtc_name']) {
				$this->op['gtc_name'] = $_POST['gtc_name'];
				$changes = true;
			}
			if($_POST['gtc_type'] && $this->op['gtc_type'] != $_POST['gtc_type']) {
				$this->op['gtc_type'] = $_POST['gtc_type'];
				$changes = true;
			}
			if($_POST['page_id'] && $this->op['page_id'] != $_POST['page_id']) {
				$this->op['page_id'] = $_POST['page_id'];
				$changePage = true;
			}
			if($_POST['gtc_interval'] && $this->op['gtc_interval'] != $_POST['gtc_interval']) {
				$this->op['gtc_interval'] = $_POST['gtc_interval'];
				$changeCron = true;
			}
			if($changes || $changeCron || $changePage) {
				update_option('gtc_options', $this->op);
				if($changeCron) {
					$this->unshedule_events_pull();
					$this->shedule_events_pull($this->op['gtc_interval']);
				}
				if($changePage){
					$post = get_post($this->op['page_id'], ARRAY_A);
					$post['post_content'] = '['.$this->eventsShortcode.'] ' . $post['post_content'];
					wp_insert_post($post);
				}
			}
		}
	}
	
	function getMessages($token, $type="notify"){
		$alert = array(
			'no-config' => 'Please fill out fields marked required.',
			'archive-alert' => 'Creating or editing event with this plugin will not affect in any way your events on Gigatools. Also be attentive when you would like to modify upcoming(future) events pulled from gigatool, these events will be overwriten periodically if you do not mark them as "archived", or are not past events already.',
			'import-succes' => '%d item(s) saved.',
			'csv-error' => 'Invalid CSV file format',
			'fopen-error' => 'Unable to open uploaded file.',
			'upload-error' => 'Upload error ocured.'
		);
		$return = '';
		if($alert[$token]){
			switch($type){
				case 'notify':
					$return = '<div class="updated">'.$alert[$token].'</div>';
					break;
				case 'error':
					$return = '<div class="error">'.$alert[$token].'</div>';
					break;
				default:
					break;
			}
		}
		return $return;
		
	}

	function gigatoolsUpcomingPage(){
		
		require_once(GTC_PLUGIN_PATH.'/lib/helper.php');
		
		$gigatoolsTable = new gigatools_List_Table();
		
		if($_REQUEST['action']==='delete' && !isset($_POST['gtc_list'])){
			if(is_numeric($_REQUEST['event-id']) && check_admin_referer('gtc-nonce')){
				$gigatoolsTable->deleteEvents($_REQUEST['event-id']);
			}
		}
		
		$gigatoolsTable->prepare_items();
		
		echo $this->getMessages('archive-alert', 'notify');
		
		?>
		<div class=wrap>
		
		<h2>Edit Gigatools events</h2>
		<h3></h3>

		<form id="gtc_events_filter" method="post">
			<input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>" />
			<input type="hidden" name="gtc_list" value="1" />

			<?php $gigatoolsTable->display() ?>
		</form>

		</div>
	<?php
	}

	function gigatoolsArchivePage(){
		require_once(GTC_PLUGIN_PATH.'/lib/helper.php');
		
		$gigatoolsTable = new gigatools_List_Table();
		
		$archive = true;
		$gigatoolsTable->prepare_items($archive);

		?>
		<div class=wrap>

		<h2>Edit Gigatools Archive</h2>
		<h3></h3>

		<form id="gtc_events_filter" method="post">
			<input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>" />
			<input type="hidden" name="gtc_list" value="1" />

			<?php $gigatoolsTable->display() ?>
		</form>

		</div>
	<?php
	}

	function gigatoolsSingleEvent(){
		require_once(GTC_PLUGIN_PATH.'/lib/helper.php');
		$gigatoolsEdit = new gigatoolsEvent();
		
		if(isset($_POST['gtc-save'])){
			$gigatoolsEdit->update();
		}
		echo $this->getMessages('archive-alert', 'notify');
		
		$gigatoolsEdit->edit();
	}

	protected function getJson($url){
		
		$ch = curl_init(); 
		curl_setopt($ch, CURLOPT_URL, $url); 
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE); 
		$json = curl_exec($ch); 
		//$httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE); 
		curl_close($ch); 

		return json_decode($json);
	}
	
	function gtcImportEvents(){
		require_once(GTC_PLUGIN_PATH.'/lib/helper.php');
		$gigatoolsEdit = new gigatoolsEvent();
		
		$gigatoolsEdit->import();
	}
	
	protected function foundInDB($group, $events, $data){
		global $wpdb;
		$ret = array();

		if($group){
			$sqlNameID = 'groupname="'.$data->group->groupname.'"';
		}else{
			$sqlNameID = 'username="'.$data->user->username.'"';
		}
		$timestamps = array();
		foreach($events as $event){
			array_push($timestamps, $event->eventdateStamp);
		}

		$sql = 'SELECT event_id, eventdate, permission, artistname 
				FROM '.$wpdb->prefix.GTC_DB_TABLE.' 
				WHERE eventdate IN ("'.implode(array_values($timestamps), '", "').'")
					AND permission="allow-update"
					AND '.$sqlNameID;
		$found = $wpdb->get_results($sql);
		foreach ($found as $key => $value) {
			$ret[$value->eventdate] = $value->event_id;
		}

		return $ret;
	}

	protected function initTable(){
		global $wpdb;
		$sql = 'CREATE TABLE IF NOT EXISTS `'.$wpdb->prefix.GTC_DB_TABLE.'` (
				`event_id` int(4) NOT NULL AUTO_INCREMENT,
				`name` varchar(255),
				`eventdate` int(11),
				`date` datetime NOT NULL DEFAULT "000-00-00 00:00:00",
				`venue` varchar(255),
				`city` varchar(75),
				`country` varchar(75),
				`info` varchar(75),
				`url_tix` varchar(255),
				`short_url` varchar(255),
				`url` varchar(255),
				`state` varchar(255),
				`showtime` varchar(255),
				`permission` varchar(20) DEFAULT "allow-update", 
				`artistname` varchar(256), 
				`username` varchar(256), 
				`groupname` varchar(256), 
				`lastupdated` int(11),
				PRIMARY KEY (`event_id`)
				) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1';
		
		return $wpdb->query($sql);
	}

	function getEventDate($date, $time){
		if(!empty($time)){
			$time = explode('T', $time);
			$time = substr($time[1], 0, 9);
			$date .= ' '.$time;
		}
		return strtotime($date);
	}

	function doThisPeriodical(){
		global $wpdb;
		$data = $this->getJson($this->getGigaUrl());
		//print_r($data);
		$group = false;
		$events = array();

		if(!empty($data)){ 
			if(!empty($data[1]) && !empty($data[0])){

				foreach ($data[1] as $key => $event){
					$e = $event->event;
					
					if(isset($data[0]->group)){
						$group = true;
						$e->groupname = $data[0]->group->groupname;
						$e->username = '';
						$e->artistname = $data[2][$key];
					}elseif(isset($data[0]->user)){
						$e->groupname = '';
						$e->username = $data[0]->user->username;
						$e->artistname = $data[0]->user->artistname;
					}

					$e->eventdateStamp = $this->getEventDate($e->eventdate, $e->showtime);
					$events[$key] = $e;
				}
				//die();
				
				//print $sql;
				$foundInDB = $this->foundInDB($group, &$events, &$data[0]);

				if(is_array($foundInDB)){
					foreach($events as $k=>$v){
						if(array_key_exists($v->eventdateStamp, $foundInDB)){
							$wpdb->update(
								$wpdb->prefix.GTC_DB_TABLE, 
								array(
									'name' 		=>$v->name,
									'venue' 	=>$v->venue,
									'city' 		=>$v->city,
									'country' 	=>$v->country,  
									'info'		=>$v->info,  
									'url_tix' 	=>$v->url_tix,  
									'short_url'	=>$v->short_url, 
									'url'		=>$v->url,
									'state'		=>$v->state,
									'showtime'	=>$v->showtime,
									'groupname'	=>$v->groupname,
									'username'	=>$v->username,
									'lastupdated'	=>time(), 
								),
								array(
									'eventdate' =>$v->eventdateStamp,
									'artistname' =>$v->artistname,
								)
							);
						}else{
							//$wpdb->show_errors();
							$wpdb->insert(
								$wpdb->prefix.GTC_DB_TABLE, 
								array(
									'eventdate' =>$v->eventdateStamp,
									'name' 	=>$v->name,
									'date' 		=>date('Y-m-d H:i:s', $v->eventdateStamp),
									'venue' 	=>$v->venue,
									'city' 		=>$v->city,
									'country' 	=>$v->country,  
									'info'		=>$v->info,  
									'url_tix' 	=>$v->url_tix,  
									'short_url' =>$v->short_url, 
									'url'		=>$v->url,
									'state'		=>$v->state,
									'showtime'	=>$v->showtime,
									'groupname' =>$v->groupname,
									'username'	=>$v->username,
									'artistname'=>$v->artistname,
									//'lastupdated'=>time(), 
								)
							);
						}
					}
				}
			}
		}
	}
	
	function upcomingEventsList($args){
		extract(shortcode_atts(array(
			'order' => 'ASC',
		), $args));
		if(!empty($_GET['y']) && is_numeric($_GET['y'])){
			$year = $_GET['y'];
			$events = $this->getEvents($year, $order);
		}else{
			$events = $this->getUpcomingEvents();
		}
		$themePath = get_template_directory().'/gtc-templates/upcomingEvents.php';
		if(file_exists($themePath)){
			include($themePath);
		}else{
			include(GTC_PLUGIN_PATH.'/gtc-templates/upcomingEvents.php');
		}
		return;
	}
	
	function yearArchive(){
		$years = $this->getYears();
		$themePath = get_template_directory().'/gtc-templates/years.php';
		if(file_exists($themePath)){
			include($themePath);
		}else{
			include(GTC_PLUGIN_PATH.'/gtc-templates/years.php');
		}
		return;
	}
	
	function getYears(){
		global $wpdb;

		$res = false;
		$sql = 'SELECT DISTINCT YEAR( date ) as year FROM `'.$wpdb->prefix.GTC_DB_TABLE.'` ORDER BY eventdate DESC';
		$res = $wpdb->get_results($sql, ARRAY_A);

		return $res;
	}

	function getEvents($year='2012', $order = 'ASC'){
		
		if($year==date('Y', time())){
			$order = 'DESC';
		}
		$year = addslashes($year);
		global $wpdb;
		$res = false;
		$sql = 'SELECT * FROM `'.$wpdb->prefix.GTC_DB_TABLE.'` WHERE YEAR( date ) = "'.$year.'" AND permission != "hidden" ORDER BY eventdate '.$order;
		$res = $wpdb->get_results($sql, ARRAY_A);

		return $res;
	}
	
	function getUpcomingEvents($order = 'ASC'){
		global $wpdb;
		$res = false;
		$sql = 'SELECT * FROM `'.$wpdb->prefix.GTC_DB_TABLE.'` WHERE eventdate > "'.time().'"  AND permission != "hidden" ORDER BY eventdate '.$order;
		$res = $wpdb->get_results($sql, ARRAY_A);

		return $res;
	}
}
}

if (class_exists("eventsPull")) {
	global $eventsPull;
	$eventsPull = new eventsPull();
	define('GTC_PLUGIN_PATH',dirname(__FILE__));
	define('GTC_DB_TABLE', 'gtc_events');
	
	//$eventsPull->doThisPeriodical();
	
	register_activation_hook(__FILE__, array($eventsPull, 'installEventsPull'));
	register_deactivation_hook(__FILE__, array($eventsPull, 'uninstallEventsPull'));
}
