<?php 
if(!class_exists('WP_List_Table')){
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

if(!class_exists('gigatools_List_Table')){
class gigatools_List_Table extends WP_List_Table {
	
	function __construct(){
		global $status, $page;
				
		//Set parent defaults
		parent::__construct( array(
			'singular'  => 'event',	 //singular name of the listed records
			'plural'	=> 'events',	//plural name of the listed records
			'ajax'	  => false		//does this table support ajax?
		) );
		
	}

	function column_default($item, $column_name){
		switch($column_name){
			case 'date':
				$echo = date('Y-m-d', $item['eventdate']);
				if($item['showtime']){
					$echo .= '<br/>'.date('H:i', $item['eventdate']);
				}
				return $echo;
				break;
			case 'venue':
			case 'city':
			case 'country':
			case 'state':
			case 'info':
			case 'short_url':
			case 'username':
			case 'groupname':
			case 'artistname':
			case 'permission':
				return $item[$column_name];
				break;
			case 'lastupdated':
				if($item[$column_name]){
					return date('Y-m-d <br/> H:i:s', $item[$column_name]);
				}
				return '';
				break;
			default:
				return $item[$column_name];
		}
	}
	
	function column_edit($item){
		
		//Build row actions
		$actions = array(
			'edit'	  => sprintf('<a href="?page=%s&action=%s&event-id=%s">Edit</a>','gtc-event','edit',$item['event_id']),
			'delete'	=> sprintf('<a href="?page=%s&gtc_list=1&action=%s&event=%s&_wpnonce=%s">Delete</a>',$_REQUEST['page'],'delete',$item['event_id'], wp_create_nonce('bulk-' . $this->_args['plural'])),
		);
		
		//Return the title contents
		return sprintf('%1$s <span style="color:silver">(id:%2$s)</span>%3$s',
			/*$1%s*/ 'Event',
			/*$2%s*/ $item['event_id'],
			/*$3%s*/ $this->row_actions($actions)
		);
	}
	
	function column_cb($item){
		return sprintf(
			'<input type="checkbox" name="%1$s[]" value="%2$s" />',
			/*$1%s*/ $this->_args['singular'],  //Let's simply repurpose the table's singular label ("event")
			/*$2%s*/ $item['event_id']				//The value of the checkbox should be the record's id
		);
	}
	
	function get_columns(){
		$columns = array(
			'cb'				=>'<input type="checkbox" />', 
			'name'				=>'Name', 
			'date'				=>'Date', 
			'venue'				=>'Venue', 
			'city'				=>'City', 
			'country'			=>'Country', 
			'state'				=>'State', 
			'info'				=>'Extra info', 
			//'url_tix'			=>'Tix URL', 
			'short_url'			=>'Short URL', 
			'username'			=>'Username', 
			'groupname'			=>'Groupname', 
			'artistname'		=>'Artist Name', 
			'lastupdated'		=>'Updated via Cron', 
			'permission'		=>'Permission / Type', 
			'edit'				=>'Edit'
		);
		return $columns;
	}
	
	function get_sortable_columns() {
		$sortable_columns = array(
			'date'	 => array('date',true),	 //true means it's already sorted
			'name'	=> array('name',false),
			'username'	=> array('username',false),
			'groupname'  => array('groupname',false),
			'artistname'  => array('artistname',false),
			'lastupdated'  => array('lastupdated',false),
		);
		return $sortable_columns;
	}
	
	function get_bulk_actions() {
		$actions = array(
			'delete'	=> 'Delete'
		);
		return $actions;
	}
	
	function process_bulk_action() {
		if(isset($_REQUEST['gtc_list'])){
			if(check_admin_referer('bulk-' . $this->_args['plural']) && 'delete'===$this->current_action() ) {
			//print_r($_POST);
				$this->deleteEvents(&$_REQUEST['event']);
				//wp_die('Items deleted (or they would be if we had items to delete)!');
			}
		}
		
	}
	function deleteEvents($events){
		global $wpdb;
		if(is_array($events)){
			$in = implode(', ',  array_values($events));
		}else{
			$in = $events;
		}
		if(!empty($in)){
			$sql = 'DELETE FROM '.$wpdb->prefix.GTC_DB_TABLE.' WHERE event_id IN ('.$in.')';
			$wpdb->query($sql);
		}
		return true;
	}
	function eventCount($archive=false){
		global $wpdb;
		if($archive){
			$where = 'WHERE `eventdate`<"'.time().'"';
		}else{
			$where = 'WHERE `eventdate`>"'.time().'"';
		}

		$sql = "SELECT count(*) as nr FROM `".$wpdb->prefix.GTC_DB_TABLE."` ".$where;
		
		$res = $wpdb->get_results($sql, ARRAY_A);

		if(is_numeric($res[0]['nr'])){
			return $res[0]['nr'];
		}else{
			return 0;
		}
	}
	
	function prepare_items($archive=false) {
		global $wpdb; 

		$per_page = 20;
		
		$columns = $this->get_columns();
		$hidden = array();
		$sortable = $this->get_sortable_columns();
		
		$this->_column_headers = array($columns, $hidden, $sortable);
		
		$this->process_bulk_action();
		
		$orderby = (!empty($_REQUEST['orderby'])) ? $_REQUEST['orderby'] : 'date';
		$order = (!empty($_REQUEST['order'])) ? $_REQUEST['order'] : 'asc';
		
		$current_page = $this->get_pagenum();

		$curentTime = time();

		$total_items = $this->eventCount($archive);

		if($archive){
			$where = 'WHERE `eventdate`<"'.time().'"';
		}else{
			$where = 'WHERE `eventdate`>"'.time().'"';
		}
	
		$sql = 'SELECT * FROM `'.$wpdb->prefix.GTC_DB_TABLE.'` '.
				$where.'  
				ORDER BY '.$orderby.' '.$order. '
				LIMIT '.(($current_page-1)*$per_page).', '.$per_page;

		$res = $wpdb->get_results($sql, ARRAY_A);

		$this->items = $res;
		
		$this->set_pagination_args( array(
			'total_items' => $total_items,				  //WE have to calculate the total number of items
			'per_page'	=> $per_page,					 //WE have to determine how many items to show on a page
			'total_pages' => ceil($total_items/$per_page)   //WE have to calculate the total number of pages
		) );
	}
	
}
}
if(!class_exists('gigatoolsEvent')){
class gigatoolsEvent{
	var $country = array(
		"", "Australia","Austria","Belgium","Brazil","Canada","France","Germany","Greece","India","Italy","Japan","Mexico","Netherlands","Poland","Portugal","Russia","Spain","Sweden","Switzerland","United Kingdom","United States","Afghanistan","Albania","Algeria","American Samoa","Andorra","Angola","Anguilla","Antigua and Barbuda","Argentina","Armenia","Aruba","Bahamas","Bahrain","Bangladesh","Barbados","Belarus","Belize","Benin","Bermuda","Bhutan","Bolivia","Bosnia Herzegowina","Botswana","Bouvet Island","Bulgaria","Burkina Faso","Burundi","Cambodia","Cameroon","Cape Verde","Cayman Islands","Central African Republic","Chad","Chile","China","Christmas Island","Cocos Islands","Colombia","Comoros","Congo","Costa Rica","Cote d'Ivoire","Croatia","Cuba","Cyprus","Czech Republic","Denmark","Djibouti","Dominica","Dominican Republic","East Timor","Ecuador","Egypt","El Salvador","Equatorial Guinea","Eritrea","Estonia","Ethiopia","Fiji","Finland","Gabon","Gambia","Georgia","Ghana","Gibraltar","Greenland","Grenada","Guadeloupe","Guam","Guatemala","Guinea","Honduras","Hong Kong","Hungary","Iceland","India","Indonesia","Iran","Iraq","Ireland","Israel","Jamaica","Jordan","Kazakhstan","Kenya","Korea","Kosovo","Lao","Latvia","Lebanon","Liechtenstein","Lithuania","Luxembourg","Macau","Macedonia","Malaysia","Malta","Marshall Islands","Martinique","Mauritania","Mauritius","Micronesia","Moldova","Monaco","Mongolia","Montenegro","Morocco","Mozambique","Namibia","Nepal","New Caledonia","New Zealand","Nicaragua","Niger","Nigeria","Norway","Oman","Pakistan","Palau","Palestine","Panama","Papua New Guinea","Paraguay","Peru","Peru","Philippines","Poland","Puerto Rico","Qatar","Romania","Russia","Saint Lucia","Samoa","San Marino","Saudi Arabia","Senegal","Serbia","Seychelles","Singapore","Slovakia","Slovenia","Solomon Islands","Somalia","South Africa","Sri Lanka","Sudan","Suriname","Swaziland","Syria","Taiwan","Tajikistan","Tanzania","Thailand","Togo","Tonga","Trinidad and Tobago","Tunisia","Turkey","Uganda","Ukraine","United Arab Emirates","Uruguay","Uzbekistan","Vanuatu","Venezuela","Vietnam","Virgin Islands","Zambia","Zimbabwe",); 

	var $state = array(
		"" => "N/A","Alabama" => "AL - Alabama","Arkansas" => "AR - Arkansas","Alaska" => "AK - Alaska","American Samoa" => "AS - American Samoa  ","Arizona" => "AZ - Arizona","California" => "CA - California","Colorado" => "CO - Colorado","Connecticut" => "CT - Connecticut","Delaware" => "DE - Delaware","District of Columbia" => "DC - District of Columbia","Florida" => "FL - Florida","Georgia" => "GA - Georgia","Guam" => "GU - Guam","Hawaii" => "HI - Hawaii","Idaho" => "ID - Idaho","Illinois" => "IL - Illinois","Indiana" => "IN - Indiana","Iowa" => "IA - Iowa","Kansas" => "KS - Kansas","Kentucky" => "KY - Kentucky","Louisiana" => "LA - Louisiana","Maine" => "ME - Maine","Marshall Islands" => "MH - Marshall Islands","Maryland" => "MD - Maryland","Massachusetts" => "MA - Massachusetts","Michigan" => "MI - Michigan","Minnesota" => "MN - Minnesota","Mississippi" => "MS - Mississippi","Missouri" => "MO - Missouri","Montana" => "MT - Montana","Nebraska" => "NE - Nebraska","Nevada" => "NV - Nevada","New Hampshire" => "NH - New Hampshire","New Jersey" => "NJ - New Jersey","New Mexico" => "NM - New Mexico","New York" => "NY - New York","North Carolina" => "NC - North Carolina","North Dakota" => "ND - North Dakota","Ohio" => "OH - Ohio","Oklahoma" => "OK - Oklahoma","Oregon" => "OR - Oregon","Palau" => "PW - Palau","Pennsylvania" => "PA - Pennsylvania","Puerto Rico" => "PR - Puerto Rico","Rhode Island" => "RI - Rhode Island","South Carolina" => "SC - South Carolina","South Dakota" => "SD - South Dakota","Tennessee" => "TN - Tennessee","Texas" => "TX - Texas","Utah" => "UT - Utah","Vermont" => "VT - Vermont","Virgin Islands" => "VI - Virgin Islands","Virginia" => "VA - Virginia","Washington" => "WA - Washington","West Virginia" => "WV - West Virginia","Wisconsin" => "WI - Wisconsin","Wyoming" => "WY - Wyoming");
	
	var $invalidFields = array();
	var $event_id = '';
	var $event = array(
		"event_id" => '',
		"eventdate" => '',
		"date" => '',
		"venue" => '',
		"city" => '',
		"country" => '',
		"info" => '',
		"url_tix" => '',
		"short_url" => '',
		"url" => '',
		"state" => '',
		"showtime" => '',
		"permission" => 'archive',
		"artistname" => '',
		"username" => '',
		"groupname" => '',
		"lastupdated" => '',
	);

	function gigatoolsEvent(){
		if (is_numeric($_REQUEST['event-id'])) {
			$this->event_id = $_REQUEST['event-id'];
		}
	}

	function getDays($current){
		$out = '<option value=""> </option>';
		for ($i=1; $i <= 31; $i++) { 
			if($i==$current) $selected = ' selected="selected"';
			$out .= '<option value="'.$i.'"'.$selected.'>'.$i.'</option>';
			$selected = '';
		}
		return $out;
	}

	function getMonths($current){
		$out = '<option value=""> </option>';
		$m = array('1'=>'January','2'=>'February','3'=>'March','4'=>'April','5'=>'May','6'=>'June','7'=>'July','8'=>'August','9'=>'September','10'=>'October','11'=>'November','12'=>'December');
		for ($i=1; $i <= 12; $i++) { 
			if($i==$current) $selected = ' selected="selected"';
			$out .= '<option value="'.$i.'"'.$selected.'>'.$m[$i].'</option>';
			$selected = '';
		}
		return $out;
	}

	function getYears($current){
		$out = '<option value=""> </option>';
		$y = date('Y', time());
		for ($i=$y-10; $i <= $y+5; $i++) { 
			if($i==$current) $selected = ' selected="selected"';
			$out .= '<option value="'.$i.'"'.$selected.'>'.$i.'</option>';
			$selected = '';
		}
		return $out;
	}

	function getMins($current){
		$out = '';
		$mins = array('', '00', '15', '30', '45');
		foreach ($mins as $v) { 
			if($v==$current) $selected = ' selected="selected"';
			$out .= '<option value="'.$v.'"'.$selected.'>'.$v.'</option>';
			$selected = '';
		}
		return $out;
	}

	function getHours($current){
		$out = '';
		$hours = array('', '00', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23');
		foreach ($hours as $v) { 
			if($v==$current) $selected = ' selected="selected"';
			$out .= '<option value="'.$v.'"'.$selected.'>'.$v.'</option>';
			$selected = '';
		}
		return $out;
	}

	function getCountry($current){
		$out = '';
		foreach ($this->country as $value) {
			if($value==$current) $selected = ' selected="selected"';
			$out .= '<option value="'.$value.'"'.$selected.'>'.$value.'</option>';
			$selected = '';	
		}
		return $out;
	}

	function getState($current){
		$out = '';
		foreach ($this->state as $k => $value) {
			if($k==$current) $selected = ' selected="selected"';
			$out .= '<option value="'.$k.'"'.$selected.'>'.$value.'</option>';
			$selected = '';	
		}
		return $out;
	}

	function getPermission($current){
		$out='';
		$perm = array(
			'allow-update'=>'update via cron',
			'archive'=>'archived',
			'hidden'=>'hidden',
		);
		foreach ($perm as $k => $v) {
			if($k==$current) $selected = ' selected="selected"';
			$out .= '<option value="'.$k.'"'.$selected.'>'.$v.'</option>';
			$selected = '';	
		}
		return $out;
 	}
	function invalidClass($v){
		if(in_array($v, $this->invalidFields)){
			return 'class="form-invalid"';
		}
		return;
	}

	function edit(){
		global $wpdb;
		$title = 'Create archive event';
		if($this->event_id){
			$title = 'Edit event';
			$sql = 'SELECT * FROM `'.$wpdb->prefix.GTC_DB_TABLE.'` WHERE `event_id`="'.$this->event_id.'"';
			$res = $wpdb->get_results($sql, ARRAY_A);
			
			if(!empty($res)){
				$this->event = $res[0];
				$title = 'Edit Gigatools event';
				$action = 'gtc-edit';		
			}else{
				return;
			}
		}
		
		include(GTC_PLUGIN_PATH.'/lib/theme/eventEdit.php');
	}
	
	function insert(){
		global $wpdb;
		if($this->validate()){
			unset($this->event['event_id']);
			$this->event['permission'] = 'archive';
			$res = $wpdb->insert(
				$wpdb->prefix.GTC_DB_TABLE, 
				$this->event
			);
			$this->event_id = $wpdb->insert_id;
			return true;
		}else{
			return false;
		}
	}
	function validate(){
		$valid = true;
		
		foreach( $_POST['event'] as $k => $v ){
			switch($k){
				case 'eventdate':
					if(!(empty($v['m']) || empty($v['d']) || empty($v['y']))){
						if(empty($v['hour'])) $v['hour'] = '00';
						if(empty($v['min'])) $v['min'] = '00';
						$date = mktime($v['hour'], $v['min'], '00', $v['m'], $v['d'], $v['y'] );
						$this->event['eventdate'] = $date;
						$this->event['date'] = date('Y-m-d H:i:s', $date);
						$this->event['showtime'] = date('c', $date);
						
						$this->removeInvalid($k);
					}else{
						$valid = false;
						array_push($this->invalidFields, $k);
					}
					break;
				case 'name':
					if(!empty($v)){
						$this->event[$k] = $v;
					}else{
						$valid = false;
						array_push($this->invalidFields, $k);
					}
				default :
					//if(!empty($v)){
						$this->event[$k] = $v;
					//}
					break;
			}
		}
		return $valid;
	}
	
	function removeInvalid($k){
		$s = array_search($k, $this->invalidFields);
		if(false !== $s){
			array_slice($this->invalidFields, $s, 1);
		}
	}
	
	function update(){
		global $wpdb;
		$update = array();
		
		if(is_numeric($_POST['event']['event_id'])){
			$this->event_id = $_POST['event']['event_id'];
		}
		
		if(is_numeric($this->event_id) && check_admin_referer('gtc-nonce')) {
			
			foreach ($_POST[event] as $k => $v) {
				if($k=='eventdate'){
					$date = mktime($v['hour'], $v['min'], '00', $v['m'], $v['d'], $v['y'] );
					$update['eventdate'] = $date;
					$update['date'] = date('Y-m-d H:i:s', $date);
					$update['showtime'] = date('c', $date);
				}else{
					$update[$k] = $v;
				}
			}
			$wpdb->update(
				$wpdb->prefix.GTC_DB_TABLE, 
				$update,
				array(
					'event_id' =>$this->event_id,
				)
			);
			return;
		}elseif(empty($this->event_id) && check_admin_referer('gtc-nonce')){
			$this->insert();	
		}
	}
	
	function import(){
	?>
		<div class=wrap>

		<h2>Import Archive</h2>
		<h3></h3>
		<?php
		if(isset($_POST['gtc-import'])){
			$this->saveImported();
		}
		?>
		
		<form id="gtc_events_filter" method="post" enctype="multipart/form-data">
			<input type="file" name="gtc-file" />
			<label>Upload a .csv file</label><br />
			<br />
			<input type="submit" value="<?php _e('Save'); ?>" name="gtc-import" id="" class="button-secondary action" />
		</form>
		Note *: You can download a sample .CSV file from <a href="https://docs.google.com/spreadsheet/pub?key=0Apw3iRuLOlg3dDVJUk9leVNJc21QZTFwNnJ5NjFWcVE&output=csv">here.</a>
		</div>
	<?php	
	}
	
	function saveImported(){
		global $wpdb, $eventsPull;
		//$wpdb->show_errors();
		$queryRes = false;
		if(isset($_FILES['gtc-file']) && $_FILES['gtc-file']['type']=='application/octet-stream' && $_FILES['gtc-file']['size']>0){
			$insert = '';
			if (($handle = fopen($_FILES['gtc-file']['tmp_name'], "r")) !== FALSE) {
				$insert = 'INSERT INTO '.$wpdb->prefix.GTC_DB_TABLE.' (`eventdate`, `date`, `name`, `venue`, `city`, `country`, `state`, `url`, `short_url`, `artistname`, `info`, `showtime`, `permission`) VALUES';
				while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
					if ($date = strtotime(str_replace('/', '.', $data[0]))) {
						$data = array_map("addslashes", $data);
						$mysqlDate = date('Y-m-d H:i:s', $date);
						$showtime = date('c', $date);
						$insert .= "('$date', '$mysqlDate', '$data[1]', '$data[2]', '$data[3]', '$data[4]', '$data[5]', '$data[6]', '$data[7]', '$data[8]', '$data[9]', '$showtime', 'archive'),";
					}
				}
				$insert = rtrim($insert, ',');
				$insert .= ';';
				$queryRes = $wpdb->query($insert);
				fclose($handle);		
			}else{
				echo $eventsPull->getMessages('fopen-error', 'error');
			}
		}else{
			echo $eventsPull->getMessages('upload-error', 'error');
		}
		if(is_numeric($queryRes)){
			echo sprintf($eventsPull->getMessages('import-succes'), $queryRes);
		}
	}
}
}
