<?php
if (!class_exists("gtc_widget")) {
class gtc_widget extends WP_Widget {

	/**
	 * Constructor
	 *
	 * @return void
	 **/
	function gtc_widget() {
		$widget_ops = array( 'classname' => 'widget_gtc_archive', 'description' => __( 'Use this widget to list year based archive of your events', 'twentyeleven' ) );
		$this->WP_Widget( 'widget_gtc_archive', 'Events Archive (gigatools)', $widget_ops );
		$this->alt_option_name = 'widget_gtc_archive';

		add_action( 'save_post', array(&$this, 'flush_widget_cache' ) );
		add_action( 'deleted_post', array(&$this, 'flush_widget_cache' ) );
		add_action( 'switch_theme', array(&$this, 'flush_widget_cache' ) );
	}

	/**
	 * Outputs the HTML for this widget.
	 *
	 * @param array An array of standard parameters for widgets in this theme
	 * @param array An array of settings for this widget instance
	 * @return void Echoes it's output
	 **/
	function widget( $args, $instance ) {
		$cache = wp_cache_get( 'widget_gtc_archive', 'widget' );

		if ( !is_array( $cache ) )
			$cache = array();

		if ( ! isset( $args['widget_id'] ) )
			$args['widget_id'] = null;

		if ( isset( $cache[$args['widget_id']] ) ) {
			echo $cache[$args['widget_id']];
			return;
		}
                
                $title = apply_filters( 'widget_title', empty( $instance['title'] ) ? __( 'Ephemera', 'twentyeleven' ) : $instance['title'], $instance, $this->id_base);

		ob_start();
		extract( $args, EXTR_SKIP );
                
                echo $before_widget;
		echo $before_title;
		echo $title; // Can set this with a widget option, or omit altogether
		echo $after_title;

		do_shortcode('[gtc_yearArchive]');		

		$cache[$args['widget_id']] = ob_get_flush();
		wp_cache_set( 'widget_gtc_archive', $cache, 'widget' );
	}

	/**
	 * Deals with the settings when they are saved by the admin. Here is
	 * where any validation should be dealt with.
	 **/
	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		$instance['title'] = strip_tags( $new_instance['title'] );
		$this->flush_widget_cache();

		$alloptions = wp_cache_get( 'alloptions', 'options' );
		if ( isset( $alloptions['widget_gtc_archive'] ) )
			delete_option( 'widget_gtc_archive' );

		return $instance;
	}

	function flush_widget_cache() {
		wp_cache_delete( 'widget_gtc_archive', 'widget' );
	}

	/**
	 * Displays the form for this widget on the Widgets page of the WP Admin area.
	 **/
	function form( $instance ) {
		$title = isset( $instance['title']) ? esc_attr( $instance['title'] ) : '';
?>
    		<p><label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php _e( 'Title:', 'twentyeleven' ); ?></label>
		<input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>" /></p>

<?php
	}
}
}